function [E,f,G,h,C,d,minmass,maxmass] = mk_verhib_matrices(soilchem,leafchem,rootchem,leafdepth,rootdepth,plantgroups,useleaf,useroot,leafrootratio,yearperlayer,regdepth,reggroups,depthlimit)
% where
%   soilchem      is the matrix with soil chemical data (nrcomp,nrlayers)
%   leafchem      is the matrix with leaf chemical data (nrcomp,nrplants)
%   rootchem      is the matrix with root chemical data (nrcomp,nrplants)
%   leafdepth     is the matrix with leaf depth relation (nrlayers,nrplants)
%   rootdepth     is the matrix with leaf depth relation (nrlayers,nrplants)
%   plantgroups   is the matrix with plant groups (nrgroups,nrplants) can be left empty
%   leafrootratio is the (1,nrplants) vector with the ratio leafmass:rootmass per plant
%   yearperlayer  is the (nrlayer,1) vector with the nr of years per depth layer
%   regdepth      is the regularization to weight the importance of stiffness over time 
%                 parameter (usually a value between 0
%                 and 0.1, smaller means less regularization)
%   reggroups     is the regularization parameter for plant groups
%   depthlimit    specifies the number of layers to consider for spatial partitioning 
%                 - to make the problem easier to sovle
%
% output-matrices and vectors are for the least squares problem:
% ||Ex - f|| subject to Gx >= h and Cx = d
%
% the plantmass is accounted for in Gx >= h
% and the leaf-root ratio in Cx = d
% 

% for a case with 2 plants, and three layers,
% and including the root info the resulting X
% matrix should look like:
%
%            leafs                     roots
%      <- nrplants*nrlayers ->   <- nrplants*nrlayers ->
%
% X = [ a1  a2  0   0   0   0    c1  c2  0   0   0   0       ^
%       b1  b2  0   0   0   0    d1  d2  0   0   0   0       |
%       0   0   a1  a2  0   0    0   0   c1  c2  0   0       | nrcomp * nrlayers
%       0   0   b1  b2  0   0    0   0   d1  d2  0   0       |
%       0   0   0   0   a1  a2   0   0   0   0   c1  c2      |
%       0   0   0   0   b1  b2   0   0   0   0   d1  d2      V
%       .9  0   0   0   0   0    .7  0   0   0   0   0       ^    
%       0   .8  0   0   0   0    0   .6  0   0   0   0       |
%       .1  0   .9  0   0   0    .2  0   .7  0   0   0       |
%       0   .2  0   .8  0   0    0   .3  0   .6  0   0       | nrplants * nrlayers
%       0   0   .1  0   .9  0    .1  0   .2  0   .7  0       | 
%       0   0   0   .2  0   .8   0   .1   0  .3  0   .6      |
%       .1  .1  0   0   0   0    .1  .1  0   0   0   0       ^
%       0   0   .1  .1  0   0    0   0   .1  .1  0   0       | nrgroup * nrlayers
%       0   0   0   0   .1  .1   0   0   0   0   .1  .1      V
%
%

if ~exist('depthlimit','var')
   depthlimit = size(leafdepth,1);
end

if ~exist('reggroups','var') || isempty(reggroups)
   reggroups = 0.1;
end

if ~exist('regdepth','var') || isempty(regdepth)
   regdepth = 0.1;
end

maxleafperyear = 1000; % (g per m2)
maxrootperyear = 1000; % (g per m2)

leafrootratio = leafrootratio(:)';

% determine the dimension of the problem
[nrcomp,nrlayers] = size(soilchem);
nrplants = size(leafchem,2);

% remove plantgroups without plants
plantsingroup = sum(plantgroups,2)>0;
plantgroups = plantgroups(plantsingroup,:);
nrgroups = size( plantgroups, 1 );
 
% leafpart = leafrootratio./(leafrootratio+1);
% rootpart = 1-leafpart;

% set values leafchem and rootchem to zero, if the respective plant
% component is not included
leafchem = leafchem .* repmat(useleaf,nrcomp,1);
rootchem = rootchem .* repmat(useroot,nrcomp,1);
if sum(rootchem(:))==0, rootchem=[]; disp('Roots not included'); end

if isempty(rootdepth) || isempty(rootchem)
    
    % build the y-vector
    ytime = zeros(nrlayers*nrplants-nrplants,1);
    
    if isempty(plantgroups)
        ygroups = [];
    else
        ygroups = reggroups * repmat(sum(plantgroups,2),nrlayers,1);
    end
    f = [ soilchem(:); ytime; ygroups];

    % build the x-matrix

    leafblocks = repmat( leafchem, nrlayers,nrlayers ) .* ...
                 spacetime_knitting( leafdepth./100, nrcomp, depthlimit );

    % the blocks to control time-stiffness distribution
    timebands = - eye(nrplants*nrlayers) + diag( ones( (nrplants*nrlayers - nrplants) ,1),-nrplants );
    timebands = timebands( nrplants+1:end , : );

    if isempty(plantgroups)
        pgroupblocks = [];
    else
        pgroupblocks = blockdiag_lim(plantgroups,nrlayers);
    end

    E = [ leafblocks; regdepth*timebands; reggroups*pgroupblocks ];

    totperlayer = blockdiag_lim( ones(1,nrplants), nrlayers );
    totleafperlayer = yearperlayer * maxleafperyear;
%     G = [ eye(nrplants*nrlayers); -totperlayer ];
%     h = [ zeros(nrplants*nrlayers,1); -totleafperlayer ];
    G = totperlayer ;
    h = totleafperlayer ;
    
    C = [];
    d = [];
    
else  % leaves and roots
    
    % build the y-vector
    ytime = zeros(2*nrlayers*nrplants - 2*nrplants,1);
    
    if isempty(plantgroups)
        ygroups = [];
    else
        ygroups = reggroups * repmat(sum(plantgroups,2),2*nrlayers,1);
    end
    
%     yleafroot = zeros(nrplants*nrlayers,1);
    
    f = [ soilchem(:); ytime; ygroups];

    
    % the blocks for the biomarker mass in the plant parts
    
    leafblocks = repmat(leafchem,nrlayers,nrlayers) .* ...
                 spacetime_knitting(leafdepth./100, nrcomp, depthlimit );
             
    rootblocks = repmat(rootchem,nrlayers,nrlayers) .* ...
                 spacetime_knitting(rootdepth./100, nrcomp, depthlimit );

    totalmass = [leafblocks rootblocks];
    
    % the blocks to control time-stiffness distribution
    timebands = - eye(nrplants*nrlayers) + diag( ones( (nrplants*nrlayers - nrplants) ,1),-nrplants );
    timebands = timebands( nrplants+1:end , : );
    zeroblock = zeros(nrplants*nrlayers - nrplants,nrplants*nrlayers);
    totaltimebands = [timebands zeroblock; zeroblock timebands];

    % the blocks for the plant groups
    if isempty(plantgroups)
        pgroupblocks = [];
    else
        pgroupblocks = blockdiag_lim(plantgroups,nrlayers);
    end
    zeroblock = zeros(nrgroups*nrlayers,nrplants*nrlayers);
    totalpgroupblocks = [pgroupblocks zeroblock; zeroblock pgroupblocks];
    
%     % the blocks for the leafroot-ratio
%     leafrootblock =  [ eye(nrplants*nrlayers),  - diag( repmat(leafrootratio',nrlayers,1) ) ];
            
    E = [ totalmass              
          regdepth *  totaltimebands 
          reggroups * totalpgroupblocks ];

    totperlayer = blockdiag_lim( ones(1,nrplants), nrlayers );
    totplantperlayer = yearperlayer * ( maxleafperyear + maxrootperyear );

%  de pijn zit hem er misschien in dat wortels en blad achter
%  elkaar staan (en gezamenlijk > dan 0 moeten zijn)
%  terwijl ze dat afzonderlijk ook zijn
%
%     G = [ eye(nrplants*nrlayers) eye(nrplants*nrlayers); 
%           -totperlayer           -totperlayer  ];
%     h = [ zeros(nrplants*nrlayers,1); -totplantperlayer ];

    G = [ totperlayer, totperlayer ];
    h = totplantperlayer ;

    C = [ eye(nrplants*nrlayers) ,  diag( repmat( -leafrootratio(:), nrlayers, 1) ) ];
    d = zeros(nrplants*nrlayers,1);

end

nrpar = size(E,2);
minmass = zeros(nrpar,1);
maxmass = ones( nrpar/nrlayers,1 ) * h' ;  % note: h is the maximum biomass production per layer
maxmass = maxmass(:);

%%
function [partitionmatrix] = spacetime_knitting(partition_in,nrcomp,nrlag)

[nrlayer, nrplant] = size(partition_in);

mindim = min(nrlayer,nrlag);

partitionmatrix = zeros(nrcomp*nrlayer,nrplant*mindim);

for diagonal = 1:mindim

    i=0;
    j=0;

    for n = 1:nrlayer + 1 - diagonal
        partition_submatrix = repmat( partition_in(diagonal,:),nrcomp,1 ) ;

        rl = (diagonal-1)*nrcomp + i + 1 : (diagonal-1)*nrcomp + i + nrcomp;
        cl = j+1:j+nrplant;
        partitionmatrix(rl,cl) = partition_submatrix ;

        i = i+nrcomp;
        j = j+nrplant;
    end

end

%%
% [output] = blockdiag_lim(input,nrblk)
%
% create a block diagonal matrix
% using nrblk blocks
%
% Emiel van Loon

function [output] = blockdiag_lim(input,nrblk)

[nrow,ncol]=size(input);
output = zeros(nrow*nrblk,ncol*nrblk);

i=0;
j=0;
for k = 1:nrblk
    output(i+1:i+nrow,j+1:j+ncol)=input;
    i = i+nrow;
    j = j+ncol;
end
